# 数据挖掘建模
   - 经过数据探索和数据预处理，一般而言此时的数据应该是可以直接建模的。
   - 根据挖掘目标和数据形式的不同，可以建立分类与预测、聚类分析、关联规则、时序模式、偏差检测等模型。
### 分类与预测
   - 如餐饮行业想知道哪些用户会离我而去，哪些用户可能成为VIP。
   - 分类和预测是预测问题的两种主要类型，分类主要是预测分类标号（离散属性），而预测主要是建立连续值函数模型，预测给定的自变量的因变量的值。
   - 分类的原理是构造一个分类模型，输入样本的属性值，必须能够输出该样本对应的类别（也叫标签）。分类模型建立在已有标记的数据集上，模型在已有样本上的准确率可以方便计算，所以分类是一种有监督的学习。
   - 预测的原理是建立两种或者两种以上的变量之间相互依赖的函数模型，然后进行预测和控制。
       - 具体的实现
           - 两者相似。
           - 分类算法大体上两个步骤。
               - 学习步。通过归纳分析训练样本集来建立分类模型得到分类规则。
               - 分类步。先用已知的测试样本集评估分类规则的准确率，如果准确率可以接受，使用该模型对未有标签的待测数据集进行预测分类。
           - 预测算法也是两个步骤。
               - 建模步。通过训练集建立预测属性（数值型）的函数模型。
               - 预测步。在模型通过检验后进行预测控制。
       - 常用的分类和预测算法
    
   | 算法名称 | 算法描述 |
   | :---: | :---: |
   | 回归分析 | 回归分析是确定预测属性（数值型）与其他变量间相互依赖的定量关系最常用的方法。包括线性回归、非线性回归、Logistic回归、岭回归、主成分回归、偏最小二乘回归等。 |
   | 决策树 | 采用自顶两下的递归方式，在内部节点进行属性值的比较，并根据不同的属性值从该节点向下分支，最终得到的叶节点是学习划分的类。 |
   | 人工神经网络 | 人工神经网络是一种模仿大脑神经网络的结构和功能而建立的信息处理系统，表示神经网络的输入和输出变量之间关系的模型。|
   | 贝叶斯网络 | 贝叶斯网络又称为信度网络，是Bayes方法的扩展， 是目前不确定知识表达和推理领域最有效的理论模型之一。 |
   | 支持向量机 | 支持向量机是一种通过某种非线性映射，把低维的非线性可分转化为高维的线性可分，在高维空间进行线性分析的算法。 |
   - 算法详解
       - 可以参考机器学习的书籍。
       - 测试代码是逻辑回归的实现，然而很多时候数据不是线性的，单纯线性回归是不合理的。
       - 决策树算法一般三个：ID3，C4.5，CART。
       - 人工神经网络（40多种模型，常用于分类和预测的有6种）实际使用要考虑拟合
   - python常用分类预测模型
        
   | 模型 | 特点 | 来源 |
   | :---: | :---: | :---: |
   | 逻辑回归 | 基础的线性分类模型，简单有效 | sklearn.linear_model |
   | SVM | 强大的模型，用来回归、预测、分类等，选取不同的核函数，模型线性或者非线性 | sklearn.svm |
   | 决策树 | 基于“分类讨论，逐步细化”思想的分类模型，直观，易解释，图形化 | skearn.tree |
   | 随机森林 | 思想类似决策树，精度更高，但是由于其随机性，丧失了决策树的可解释性 | sklearn.ensemble |
   | 朴素贝叶斯 | 基于概率思想的简单有效的分类模型，给出容易理解的概率解释 | sklearn.naive_bayes |
   | 神经网络 | 强大的拟合能力，用于拟合、分类等，很多强化版本如递神经网络，卷积神经网络，自编码器等，这些是深度学习的模型基础 | Keras |
### 聚类分析
  - 群体上的考究。
   - 和分类不同，这里所有的数据都是事先没有类别的。
   - 这是非监督的学习算法。
   - 原理：给定一组数据，根据数据自身距离或者相似度将其划分为若干组，原则为组内距离最小而组间距离最大。
   - 常用聚类方法。
      - 划分方法（分裂）：K-Means算法（K-平均）、K-MEDOIDS算法（K-中心点）、CLARANS算法（基于选择的算法）
      - 层次分析方法：BIRCH算法（平均迭代规约和聚类）、CURE算法（代表点聚类）、CHAMELEON算法（动态模型）
      - 基于密度的方法：DBSCAN算法（基于高密度连接区域）、DENCLUE算法（密度分布函数）、OPTICS算法（对象排序识别)
      - 基于网格的方法：STING算法（统计信息网络）、CLIOUE算法（聚类高维空间）、WAVE-CLUSTER算法（小波变换）
      - 基于模型的方法：统计学方法、神经网络方法
    - 常用聚类分析算法
    
   | 算法名称 | 算法描述 |
   | :---: | :--- |
   | K-Means | K-均值聚类也叫作快速聚类法，在最小化误差函数的基础上将数据划分为预定的函数K。原理简单，适合处理大量数据。 |
   | K-中心点 | K-Means对于孤立点的敏感性，K-中心点不使用簇中对象的平均值作为簇中心，而选用簇中离平均值最近的对象作为簇中心。 |
   | 系统聚类 | 也叫作多层次聚类，分类的单位由高到低呈现树形结构，所处位置越低，包含对象越少，但这些对象的共同特征越多。这种聚类方法只适合小数据量，会比较慢。 |
   
### 关联规则
   - 数据之间是有关系的互斥或者依存。（淘宝推荐，百度广告）
   - 关联规则分析是数据挖掘领域最活跃的研究方法，目的是在一个数据集中找出各项之间的关联联系，而这种联系是数据没有表示的。
   - 常用关联规则算法。
    
   | 算法名称 | 算法描述 |
   | :---: | :--- |
   | Apriori算法 | 关联规则最常用也是最经典的挖掘频繁项集的算法，核心思想是通过连接产生候选项及其支持度然后通过剪枝生成频繁项集。 |
   | FP-Tree算法 | 针对Apriori固有的多次扫描事物数据集的缺陷，提出的不产生候选频繁项集的算法。与前者都是寻找频繁项集的算法。 |
   | Eclat算法 | 深度优先算法，采用垂直数据表示形式，在概念格理论的基础上利用基于前缀的等价关系将搜索空间划分为较小的子空间。 |
   | 灰色关联法 | 分析和确定各元素之间的影响程度或者是若干个子元素（子序列）对主元素（母序列）的贡献度而进行的一种分析方法。 |
   
### 时序模式
   - 从已有的时间数据推测未来的数据有时候是必要的。
   - 算法模型较多。
   - 主要有AR模型，MA模型，ARMA模型，ARIMA模型。
   - 需要预处理。（随机性和平稳性检测）
### 离群点检测
   - 异常检测。
   - 异常值丢弃或者异常值获取。
   - 检测方法。
       - 基于模型检测。
       - 基于聚类检测。

                   
                       